#!/bin/bash

VERSION="0.9"
UTVERSION="436"

clear
echo
echo "Welcome to the Unreal Tournament $UTVERSION installer for Linux"
echo "Version $VERSION"
echo "written by abfackeln <abfackeln@abfackeln.com>"
echo

export LANG=C

MY_PATCH_FILE=ut-install-$UTVERSION.run
MY_PATCH_INPUT=install$UTVERSION-std.input
MY_PATCH2_FILE=ut-patch-$UTVERSIONa.run
MY_PATCH2_INPUT=install$UTVERSIONa-std.input

if [ ! -f patches/$MY_PATCH_FILE ]; then
	echo "patch file $MY_PATCH_FILE not found in the patches directory!"
	echo "unable to continue. (make sure file is all lowercase)"
	echo "file can be downloaded from http://unreal.epicgames.com/TournamentVersions.htm"
	exit 0
fi

echo
echo "Please take time now to answer a few quick questions to configure your server"
echo
echo "You can not use the pipe (|) character in your answers."
echo "Enter ctrl-c to cancel installation."
echo
echo
echo "if you are using a copy of the CDROM, then the CDROM directory you specify"
echo "must contain _at_least_ the System directory from the CDROM."
echo

### locate the CDROM, and export the value

echo -n "Enter the path to the CDROM (or a copy of it): "
read SETUP_CDROM
export SETUP_CDROM
echo

if [ ! -d $SETUP_CDROM/System ]; then
	echo "the unreal cdrom must be mounted at $SETUP_CDROM"
	echo "unable to continue."
	exit 0
fi

### get the install directory and the destination directory

INSTALL_DIR=`pwd`

echo "The directory which you install should be the FULL pathname"
echo "and should not already exist, ie. /usr/local/unreal"
echo
MYDIR=
while [ -z "$MYDIR" ]; do
	echo -n "Enter the directory where you wish to install Unreal Tournament: "
	read MYDIR
	if [ ! -d "$MYDIR" ]; then
		mkdir "$MYDIR"
	else
		echo "Directory already exists."
		echo "can not continue because this will destroy the directory you specified."
		echo "if you really want to use this directory name, delete the existing one first"
		MYDIR=
	fi
done

if [ ! -d "$MYDIR" ]; then
	echo "Unable to create directory."
	exit 0
fi
cd "$MYDIR"

### create a file /tmp/ut.ini to work on UnrealTournament.ini settings

cp -pf $SETUP_CDROM/System/UnrealTournament.ini /tmp/ut.ini

### replace lines in the UnrealTournament.ini file

sed -f $INSTALL_DIR/custom/common.sed /tmp/ut.ini > /tmp/ut2.ini
cp -pf /tmp/ut2.ini /tmp/ut.ini

### add custom .ini settings from the more-ini.txt file

cat $INSTALL_DIR/custom/more-ini.txt >> /tmp/ut.ini

### ask some questions

echo
echo "if you intend to run this program as a user other than root, then enter"
echo "the username here .. if you run as root then it will suid to this user"
echo
USERID=
while [ -z "$USERID" ]; do
	echo -n "Enter the userid that the ucc program should run as [nobody]: "
	read USERID
	if [ "$USERID" = "" ]; then
		USERID="nobody"
	fi
	if [ "$USERID" = "root" ]; then
		echo "invalid user id; should not run as root, pick someone else"
		USERID=
	fi
	XXTMP=`grep ^$USERID: /etc/passwd`
	if [ "$XXTMP" = "" ]; then
		echo "invalid user id"
		USERID=
	fi
	XXTMP=
done

echo
INST_CLIENT=
while [ -z "$INST_CLIENT" ]; do
	echo -n "Would you like to install the UT client (y/n): "
	read VAL
	if [ "$VAL" = "y" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "Y" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "YES" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "YEs" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "Yes" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "yes" ]; then
		INST_CLIENT="y"
	fi
	if [ "$VAL" = "n" ]; then
		INST_CLIENT="n"
	fi
	if [ "$VAL" = "N" ]; then
		INST_CLIENT="n"
	fi
	if [ "$VAL" = "No" ]; then
		INST_CLIENT="n"
	fi
	if [ "$VAL" = "NO" ]; then
		INST_CLIENT="n"
	fi
	if [ "$VAL" = "no" ]; then
		INST_CLIENT="n"
	fi
done

echo
GENBOTS=
while [ -z "$GENBOTS" ]; do
	echo -n "Would you like to give the bots binary names (y/n): "
	read VAL
	if [ "$VAL" = "y" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "Y" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "YES" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "YEs" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "Yes" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "yes" ]; then
		GENBOTS="y"
	fi
	if [ "$VAL" = "n" ]; then
		GENBOTS="n"
	fi
	if [ "$VAL" = "N" ]; then
		GENBOTS="n"
	fi
	if [ "$VAL" = "No" ]; then
		GENBOTS="n"
	fi
	if [ "$VAL" = "NO" ]; then
		GENBOTS="n"
	fi
	if [ "$VAL" = "no" ]; then
		GENBOTS="n"
	fi
done
if [ "$GENBOTS" = "y" ]; then
	echo "storing old botfile as System/User.ini.old-bots in case you want it later"
else
	echo "writing binary botfile System/User.ini.binary-bots in case you want it later"
fi

echo
LOCAL_STATS=
while [ -z "$LOCAL_STATS" ]; do
	echo -n "Would you like local stats enabled (requires JDK 1.2) (y/n): "
	read VAL
	if [ "$VAL" = "y" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "Y" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "YES" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "YEs" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "Yes" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "yes" ]; then
		LOCAL_STATS="y"
	fi
	if [ "$VAL" = "n" ]; then
		LOCAL_STATS="n"
	fi
	if [ "$VAL" = "N" ]; then
		LOCAL_STATS="n"
	fi
	if [ "$VAL" = "No" ]; then
		LOCAL_STATS="n"
	fi
	if [ "$VAL" = "NO" ]; then
		LOCAL_STATS="n"
	fi
	if [ "$VAL" = "no" ]; then
		LOCAL_STATS="n"
	fi
done
if [ "$LOCAL_STATS" = "y" ]; then
	sed "s|^bBatchLocal=.*|bBatchLocal=True|" /tmp/ut.ini > /tmp/ut2.ini
	cp -pf /tmp/ut2.ini /tmp/ut.ini
fi

echo
VAL=
while [ -z "$VAL" ]; do
	echo -n "Enter a name for your server: "
	read VAL
done
sed "s|^ServerName=.*|ServerName=$VAL|" /tmp/ut.ini > /tmp/ut2.ini
cp -pf /tmp/ut2.ini /tmp/ut.ini

echo
VAL=
while [ -z "$VAL" ]; do
	echo -n "Enter a username for the Web admin tool: "
	read VAL
done
sed "s|^AdminUsername=.*|AdminUsername=$VAL|" /tmp/ut.ini > /tmp/ut2.ini
cp -pf /tmp/ut2.ini /tmp/ut.ini

echo
VAL=
while [ -z "$VAL" ]; do
	echo -n "Enter a password for the web admin tool and the console adminlogin: "
	read VAL
done
sed "s|^AdminPassword=.*|AdminPassword=$VAL|" /tmp/ut.ini > /tmp/ut2.ini
cp -pf /tmp/ut2.ini /tmp/ut.ini

echo
echo -n "Enter the name of the server administrator (Optional): "
read VAL
sed "s|^AdminName=.*|AdminName=$VAL|" /tmp/ut.ini > /tmp/ut2.ini
cp /tmp/ut2.ini /tmp/ut.ini

echo
echo -n "Enter an administrator email address (Optional): "
read VAL
sed "s|^AdminEmail=.*|AdminEmail=$VAL|" /tmp/ut.ini > /tmp/ut2.ini
cp /tmp/ut2.ini /tmp/ut.ini

LINE1="Installed with abfackeln's Linux UT installer"
LINE2="http://abfackeln.com/info/ut-installer.html"
LINE3="To modify this text, go to this url:"
LINE4="http://$HOSTNAME:8888/"

sed "s|^MOTDLine1=.*|MOTDLine1=$LINE1|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^MOTDLine2=.*|MOTDLine2=$LINE2|" /tmp/ut2.ini > /tmp/ut.ini
sed "s|^MOTDLine3=.*|MOTDLine3=$LINE3|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^MOTDLine4=.*|MOTDLine4=$LINE4|" /tmp/ut2.ini > /tmp/ut.ini

echo
echo
echo "Thank you!"
echo "Now, go have a coffee .. the rest of the install is automatic ......"
echo



## now work on installing stuff ..



echo "copying files from the cd ..."

# copy dos version to our new directory
#cp -pR $SETUP_CDROM/Help .
#cp -pR $SETUP_CDROM/Logs .
#cp -pR $SETUP_CDROM/Maps .
#cp -pR $SETUP_CDROM/Music .
#cp -pR $SETUP_CDROM/NetGamesUSA.com .
#cp -pR $SETUP_CDROM/Sounds .
cp -pR $SETUP_CDROM/System .
#cp -pR $SETUP_CDROM/Textures .
#cp -pR $SETUP_CDROM/Web .

# remove dos-specific files
find . -name "*.dll" -exec rm {} \; 
find . -name "*.exe" -exec rm {} \; 
find . -name "*.EXE" -exec rm {} \; 
find . -name "*.frt" -exec rm {} \; 
find . -name "*.itt" -exec rm {} \; 
find . -name "*.est" -exec rm {} \; 

# install the patch -- this currently only installs the server files

echo
echo "Installing patch from $MY_PATCH_FILE ..."
echo
echo

if [ "$INST_CLIENT" = "y" ]; then
	CLIENT_OPENGL=y
	CLIENT_GLIDE=y
	CLIENT_DESKTOP=y
else
	CLIENT_OPENGL=n
	CLIENT_GLIDE=n
	CLIENT_DESKTOP=n
fi

sed -e "s|@@MYDIR@@|$MYDIR|" -e "s|@@CLIENT_OPENGL@@|$CLIENT_OPENGL|" -e "s|@@CLIENT_GLIDE@@|$CLIENT_GLIDE|" -e "s|@@CLIENT_DESKTOP@@|$CLIENT_DESKTOP|" $INSTALL_DIR/custom/$MY_PATCH_INPUT > /tmp/install.input
sh $INSTALL_DIR/patches/$MY_PATCH_FILE < /tmp/install.input
echo
echo

# install second patch if necessary

echo
if [ -f $INSTALL_DIR/patches/$MY_PATCH2_FILE ]; then
	echo "Installing patch from $MY_PATCH2_FILE ..."
	sed -e "s|@@MYDIR@@|$MYDIR|" $INSTALL_DIR/custom/$MY_PATCH2_INPUT > /tmp/install.input
	sh $INSTALL_DIR/patches/$MY_PATCH2_FILE < /tmp/install.input
else
	echo "file not found: $MY_PATCH2_FILE"
	echo "secondary patch file is not detected -- no problem."
	echo "$MY_PATCH_FILE should be enough."
fi
echo
echo
rm -f /tmp/install.input

# install bonus pack if the file exists

echo
echo "Looking for bonus packs ..."

if [ -f $INSTALL_DIR/patches/ut-bonuspack.tar.gz ]; then
	echo
	echo "Installing Bonus Pack from ut-bonuspack.tar.gz ..."
	tar zxpf $INSTALL_DIR/patches/ut-bonuspack.tar.gz
fi

if [ -d UT-BonusPack-0.1-beta ]; then
	cp -pf UT-BonusPack-0.1-beta/System/* System
	cp -pf UT-BonusPack-0.1-beta/Textures/* Textures
	cp -pf UT-BonusPack-0.1-beta/Maps/* Maps
	cp -pf UT-BonusPack-0.1-beta/Help/* Help

	echo
	echo "fixing .ini settings that umod should have set but did not."
	sed -f $INSTALL_DIR/custom/bonuspack.sed < /tmp/ut.ini > /tmp/ut2.ini
	mv -f /tmp/ut2.ini /tmp/ut.ini

	echo "removing bonuspack directory";
	rm -rf UT-BonusPack-0.1-beta
else

# proper bonus pack installation -- the above one works, but requires a special tar file
# this next one uses the standard epic umod in a zip file

	if [ -f $INSTALL_DIR/patches/utbonuspack.zip ]; then
		echo
		echo "Installing Bonus Pack from utbonuspack.zip ..."
		unzip -o $INSTALL_DIR/patches/utbonuspack.zip
		cd $INSTALL_DIR/custom/umodpack-abf
		./umod -b $MYDIR -f -i $MYDIR/UTBonusPack.umod
		cd $MYDIR
		rm -f UTBonusPack.umod

		echo
		echo "Fixing Bonus Pack case-insensitivity bug."
		#mv -fv system/* System
		#mv -fv textures/* Textures
		#rmdir system textures
		cd System
		mv -fv relics.int Relics.int
		mv -fv relics.u Relics.u
		mv -fv relicsbindings.int RelicsBindings.int
		mv -fv relicsbindings.u RelicsBindings.u
		mv -fv multimesh.int MultiMesh.int
		mv -fv multimesh.u MultiMesh.u
		mv -fv epiccustommodels.u EpicCustomModels.u
		mv -fv tnalimeshskins.int TNaliMeshSkins.int
		mv -fv tskmskins.int TSkMSkins.int
		mv -fv tcowmeshskins.int TCowMeshSkins.int
		cd ../Textures
		mv -fv tnalimeshskins.utx TNaliMeshSkins.utx
		mv -fv tskmskins.utx TSkMSkins.utx
		mv -fv tcowmeshskins.utx TCowMeshSkins.utx
		cd ..

		echo
		echo "fixing .ini settings that umod should have set but did not."
		sed -f $INSTALL_DIR/custom/bonuspack.sed < /tmp/ut.ini > /tmp/ut2.ini
		mv -f /tmp/ut2.ini /tmp/ut.ini
	fi
fi

# install the inoxx bonus pack if the file exists.

if [ -f $INSTALL_DIR/patches/utinoxxpack.zip ]; then
	echo
	echo "Installing Inoxx Bonus Pack from utinoxxpack.zip ..."
	unzip -o $INSTALL_DIR/patches/utinoxxpack.zip
	cd $INSTALL_DIR/custom/umodpack-abf
	./umod -b $MYDIR -f -i $MYDIR/UTInoxxPack.umod
	cd $MYDIR
	rm -f UTInoxxPack.umod
fi

# # # install NetGamesUSA 1.5.6 and destroy the packaged copy
# # 
# # echo
# # echo "Fixing NetGames WorldStats .. installing version 1.5.6 ..."
# # 
# # #rm -rf NetGamesUSA.com
# # #rm -f NetGamesUSA.com.tar
# # mv -f NetGamesUSA.com NetGamesUSA.com.loki
# # 
# # tar zxpf $INSTALL_DIR/patches/netgamesusa.com-1.5.6.tar.gz

# customize the UnrealTournament.ini file and create init script
# make a copy for each game type

echo
echo "Modifying UnrealTournament.ini file ..."

# DM

MYID="7777"
MYWEB="7776"
MYGAME="DeathMatchPlus"
MYMAP="DM-Barricade"
ABBR="DM"

cp -f $INSTALL_DIR/custom/ucc.init /tmp/ucc.init
sed "s|^ABBR=.*|ABBR=$ABBR|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYDIR=.*|MYDIR=$MYDIR|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYGAME=.*|MYGAME=$MYGAME|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYMAP=.*|MYMAP=$MYMAP|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYUSERID=.*|MYUSERID=$USERID|" /tmp/ucc.init > ./ucc-$ABBR.init
rm -f /tmp/ucc.init /tmp/ucc2.init
chmod u+x ucc-$ABBR.init
VAL2=$[$MYID+3]
sed "s|^MOTDLine4=.*|MOTDLine4=http://$HOSTNAME:$MYWEB/|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^ListenPort=.*|ListenPort=$MYWEB|" /tmp/ut2.ini > /tmp/ut3.ini
sed "s|^DefaultServerGame=.*|DefaultServerGame=Botpack.$MYGAME|" /tmp/ut3.ini > /tmp/ut2.ini
sed "s|^Port=.*|Port=$MYID|" /tmp/ut2.ini > /tmp/ut-$ABBR.ini

# CTF

MYID="7767"
MYWEB="7766"
MYGAME="CTFGame"
MYMAP="CTF-Coret"
ABBR="CTF"

cp -f $INSTALL_DIR/custom/ucc.init /tmp/ucc.init
sed "s|^ABBR=.*|ABBR=$ABBR|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYDIR=.*|MYDIR=$MYDIR|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYGAME=.*|MYGAME=$MYGAME|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYMAP=.*|MYMAP=$MYMAP|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYUSERID=.*|MYUSERID=$USERID|" /tmp/ucc.init > ./ucc-$ABBR.init
rm -f /tmp/ucc.init /tmp/ucc2.init
chmod u+x ucc-$ABBR.init
VAL2=$[$MYID+3]
sed "s|^MOTDLine4=.*|MOTDLine4=http://$HOSTNAME:$MYWEB/|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^ListenPort=.*|ListenPort=$MYWEB|" /tmp/ut2.ini > /tmp/ut3.ini
sed "s|^DefaultServerGame=.*|DefaultServerGame=Botpack.$MYGAME|" /tmp/ut3.ini > /tmp/ut2.ini
sed "s|^Port=.*|Port=$MYID|" /tmp/ut2.ini > /tmp/ut-$ABBR.ini

# DOM

MYID="7757"
MYWEB="7756"
MYGAME="Domination"
MYMAP="DOM-Cinder"
ABBR="DOM"

cp -f $INSTALL_DIR/custom/ucc.init /tmp/ucc.init
sed "s|^ABBR=.*|ABBR=$ABBR|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYDIR=.*|MYDIR=$MYDIR|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYGAME=.*|MYGAME=$MYGAME|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYMAP=.*|MYMAP=$MYMAP|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYUSERID=.*|MYUSERID=$USERID|" /tmp/ucc.init > ./ucc-$ABBR.init
rm -f /tmp/ucc.init /tmp/ucc2.init
chmod u+x ucc-$ABBR.init
VAL2=$[$MYID+3]
sed "s|^MOTDLine4=.*|MOTDLine4=http://$HOSTNAME:$MYWEB/|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^ListenPort=.*|ListenPort=$MYWEB|" /tmp/ut2.ini > /tmp/ut3.ini
sed "s|^DefaultServerGame=.*|DefaultServerGame=Botpack.$MYGAME|" /tmp/ut3.ini > /tmp/ut2.ini
sed "s|^Port=.*|Port=$MYID|" /tmp/ut2.ini > /tmp/ut-$ABBR.ini

# AS

MYID="7747"
MYWEB="7746"
MYGAME="Assault"
MYMAP="AS-Frigate"
ABBR="AS"

cp -f $INSTALL_DIR/custom/ucc.init /tmp/ucc.init
sed "s|^ABBR=.*|ABBR=$ABBR|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYDIR=.*|MYDIR=$MYDIR|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYGAME=.*|MYGAME=$MYGAME|" /tmp/ucc.init > /tmp/ucc2.init
sed "s|^MYMAP=.*|MYMAP=$MYMAP|" /tmp/ucc2.init > /tmp/ucc.init
sed "s|^MYUSERID=.*|MYUSERID=$USERID|" /tmp/ucc.init > ./ucc-$ABBR.init
rm -f /tmp/ucc.init /tmp/ucc2.init
chmod u+x ucc-$ABBR.init
VAL2=$[$MYID+3]
sed "s|^MOTDLine4=.*|MOTDLine4=http://$HOSTNAME:$MYWEB/|" /tmp/ut.ini > /tmp/ut2.ini
sed "s|^ListenPort=.*|ListenPort=$MYWEB|" /tmp/ut2.ini > /tmp/ut3.ini
sed "s|^DefaultServerGame=.*|DefaultServerGame=Botpack.$MYGAME|" /tmp/ut3.ini > /tmp/ut2.ini
sed "s|^Port=.*|Port=$MYID|" /tmp/ut2.ini > /tmp/ut-$ABBR.ini

rm -f /tmp/ut3.ini /tmp/ut2.ini /tmp/ut.ini

# copy the ini file and a backup and set up different game links

cd System
cp -pvf $SETUP_CDROM/System/UnrealTournament.ini UnrealTournament.ini.orig
mv -vf /tmp/ut-DM.ini UnrealTournament-DM.ini
mv -vf /tmp/ut-CTF.ini UnrealTournament-CTF.ini
mv -vf /tmp/ut-AS.ini UnrealTournament-AS.ini
mv -vf /tmp/ut-DOM.ini UnrealTournament-DOM.ini
ln -sf UnrealTournament-DM.ini UnrealTournament.ini
ln -sf ucc-bin ucc-DM
ln -sf ucc-bin ucc-CTF
ln -sf ucc-bin ucc-AS
ln -sf ucc-bin ucc-DOM
cd ..

### set bots to autoadjust skill .. this is generally best for everyone

sed 's|^bAdjustSkill=[A-Za-z]*|bAdjustSkill=True|' System/User.ini > /tmp/User.ini
mv -f /tmp/User.ini System/User.ini

## generic bot skins and names

if [ "$GENBOTS" = "y" ]; then
	cp -p System/User.ini System/User.ini.old-bots
	$INSTALL_DIR/custom/generic-bots.sh System/User.ini.old-bots > System/User.ini
else
	$INSTALL_DIR/custom/generic-bots.sh System/User.ini > System/User.ini.binary-bots
fi

# make the nobody user have full access to everything

chown -R $USERID .
chmod u+s System/ucc-bin

if [ -f $INSTALL_DIR/patches/BotPack.u ]; then
	cp -f $INSTALL_DIR/patches/BotPack.u System
fi

# all done .. print info and set up shortcuts

echo
echo
echo
echo
echo "Installation Complete"
echo
echo "to start the server enter the command:"
echo "$MYDIR/ucc-DM.init start"
echo

if [ "$INST_CLIENT" = "y" ]; then
	mv /tmp/ut /usr/local/bin
	echo "to start the client enter the command:"
	echo "ut"
	echo
fi
rm -f /tmp/ucc /tmp/ut /tmp/upgrade.log

# EOF
